window.Gleam.initSearch([{"doc":"funsies","title":"funsies","content":"# Funsies\n\n&gt; A fun, friendly, and type-safe ORM for Gleam! Compose type-safe SQL queries, generate decoders and types from schemas, and with a fun CLI!\n\n## About\n\nFunsies is a type-safe ORM for Gleam. Obviously, there are other database tools for Gleam. But, I found that most of them felt too barebones for my taste. So, I built the ORM of my dreams.\n\nFunsies comes with a DSL for schema creation (these schemas are used throughout the rest of the ORM), a DSL for building queries (with an innovative, stack-based approach), and a CLI for generating boilerplate code from your schemas. All in one, simple package.\n\n## Installation\n\n```bash\ngleam add funsies\n```\n\n## Usage\n\n&gt; THIS PART OF THE DOCS IS IN PROGRESSSSSSS\n\nTo define a schema, create a `schema` folder in your `src` directory. Then, create a `{name}.gleam` file in there, where `{name}` is the name of the schema you&#39;re creating. For example, I&#39;d create a `user.gleam` file with the following inside of it:\n\n```gleam\npub fn user() {\n  schema.create_table(&quot;user&quot;)\n  |&gt; schema.add_serial_column(&quot;id&quot;)\n  |&gt; schema.add_string_column(&quot;name&quot;, 255)\n  |&gt; schema.add_bool_column(&quot;is_active&quot;)\n}\n```\n\nThis is where the magic happens! This schema is the brains of your application. It tells Funsies how to create queries, type-check them, and generate pitch-perfect decoders. All with this little, **reusable** definition.\n\nTo create the necessary types and decoders, run the following command:\n\n```bash\ngleam run -m funsies\n```\n\nAnd thats it! You will now have decoders and types for your schema, as well as an `insert` command that is type-safe and ensures you are passing the correct values in the correct order. The decoders are used from the postgres driver to decode the results of your queries into the generated types. That way, you get easy-to-work-with and type-safe data.\n\n&gt; The ORM/query builder is a work in progress. Below is what we CURRENTLY have.\n\nTo create a query, you can use the `yummy` DSL.\n\nFor example:\n\n```gleam\nimport funsies/query/yummy as y\nimport schema/users\nimport gleam/io.{debug}\n\npub fn main() {\n  let table = users.users()\n\n  let query =y.new(table)\n  |&gt; y.select()\n  |&gt; y.where(\n    y.wb(table)\n    |&gt; y.equals(&quot;name&quot;, &quot;Alice&quot;)\n    |&gt; y.not()\n    |&gt; y.equals(&quot;id&quot;, 4)\n    |&gt; y.and()\n    |&gt; y.equals(&quot;id&quot;, 5)\n    |&gt; y.or()\n  )\n  |&gt; y.to_sql()\n\n  case query {\n    Ok(sql) =&gt; debug(sql)\n    Error(_) =&gt; debug(&quot;Error!&quot;)\n  }\n}\n```\n\nThis might look a bit weird at first. Don&#39;t worry! It is natural. Instead of usual composition of functions, we use a stack-based DSL. This allows for a neater, more readable syntax. This yummy query would look like this in a more traditional DSL:\n\n```gleam\ny.or(y.and(y.not(y.equals(&quot;name&quot;, &quot;Alice&quot;)), y.equals(&quot;id&quot;, 4)), y.equals(&quot;id&quot;, 5))\n```\n\nNow, you might see the issue with this &quot;traditional&quot; syntax. It is HARD. TO. READ! Why have an ugly syntax in such a beautiful language?\n\nSo, we use the stack-based DSL. It may take a bit of getting used to, but it&#39;s worth it!\n\nThere are other benefits to our DSL. For example, we have full type-safety. This is because we use the types directly in the DSL to build the query. This means that if you try to add a `String` where a `Bool` is expected, the type-checker will catch it. By expected I mean the type of the column that you are referencing (EVEN THOUGH YOU ARE JUST PASSING A STRING! Magic of the schema :D)\n\nThis could look like:\n\n```gleam\n|&gt; y.equals(&quot;name&quot;, 1) // Error, 1 is not a string. It knows that the name value of the column is a string, so it can&#39;t be compared to an int.\n|&gt; y.equals(&quot;is_active&quot;, &quot;true&quot;) // Error, &quot;true&quot; is not a bool\n```\n\nThis will be propogated throughout, and you will get an `Error` value for the query instead of an `Ok` value.\n\nWhile you can still make invalid queries with the yummy DSL, you will not be able to make some common errors (i.e. passing the wrong type of value). This is because funsies checks the schema, sees that the column named &quot;name&quot; is a `String` column, and also sees that you are trying to pass an `Int` to an `equals`. Thus, it will return an error! Pretty cool if I do say so myself 🤓\n\nYou can also insert values.\n","url":"index.html"},{"doc":"funsies/schema","title":"Column","content":"pub type Column {\n  StringColumn(name: String, size: Int)\n  IntColumn(name: String)\n  BoolColumn(name: String)\n  ForeignKeyColumn(\n    name: String,\n    references_table: String,\n    references_column: String,\n    references_type: String,\n  )\n  SerialColumn(name: String)\n}\n\nStringColumn(name: String, size: Int)\n\n\nIntColumn(name: String)\n\n\nBoolColumn(name: String)\n\n\nForeignKeyColumn(\n  name: String,\n  references_table: String,\n  references_column: String,\n  references_type: String,\n)\n\n\nSerialColumn(name: String)\n\n\nSynonyms:\nfunsies/schema.Column\nfunsies/schema Column","url":"funsies/schema.html#Column"},{"doc":"funsies/schema","title":"Table","content":"pub type Table {\n  Table(name: String, columns: List(Column))\n}\n\nTable(name: String, columns: List(Column))\n\n\nSynonyms:\nfunsies/schema.Table\nfunsies/schema Table","url":"funsies/schema.html#Table"},{"doc":"funsies/schema","title":"add_bool_column","content":"pub fn add_bool_column(table: Table, name: String) -&gt; Table\n\nSynonyms:\nfunsies/schema.add_bool_column\nfunsies/schema add_bool_column","url":"funsies/schema.html#add_bool_column"},{"doc":"funsies/schema","title":"add_foreign_key_column","content":"pub fn add_foreign_key_column(\n  table: Table,\n  name: String,\n  references_table: Table,\n  references_column: String,\n) -&gt; Table\n\nSynonyms:\nfunsies/schema.add_foreign_key_column\nfunsies/schema add_foreign_key_column","url":"funsies/schema.html#add_foreign_key_column"},{"doc":"funsies/schema","title":"add_int_column","content":"pub fn add_int_column(table: Table, name: String) -&gt; Table\n\nSynonyms:\nfunsies/schema.add_int_column\nfunsies/schema add_int_column","url":"funsies/schema.html#add_int_column"},{"doc":"funsies/schema","title":"add_serial_column","content":"pub fn add_serial_column(table: Table, name: String) -&gt; Table\n\nSynonyms:\nfunsies/schema.add_serial_column\nfunsies/schema add_serial_column","url":"funsies/schema.html#add_serial_column"},{"doc":"funsies/schema","title":"add_string_column","content":"pub fn add_string_column(\n  table: Table,\n  name: String,\n  size: Int,\n) -&gt; Table\n\nSynonyms:\nfunsies/schema.add_string_column\nfunsies/schema add_string_column","url":"funsies/schema.html#add_string_column"},{"doc":"funsies/schema","title":"create_table","content":"pub fn create_table(name: String) -&gt; Table\n\nSynonyms:\nfunsies/schema.create_table\nfunsies/schema create_table","url":"funsies/schema.html#create_table"},{"doc":"funsies/schema","title":"generate_create_table_sql","content":"pub fn generate_create_table_sql(table: Table) -&gt; String\n\nSynonyms:\nfunsies/schema.generate_create_table_sql\nfunsies/schema generate_create_table_sql","url":"funsies/schema.html#generate_create_table_sql"},{"doc":"funsies/schema","title":"generate_drop_table_sql","content":"pub fn generate_drop_table_sql(table: Table) -&gt; String\n\nSynonyms:\nfunsies/schema.generate_drop_table_sql\nfunsies/schema generate_drop_table_sql","url":"funsies/schema.html#generate_drop_table_sql"},{"doc":"funsies/schema","title":"get_part_table","content":"pub fn get_part_table(\n  table: Table,\n  columns: List(String),\n) -&gt; Table\n\nSynonyms:\nfunsies/schema.get_part_table\nfunsies/schema get_part_table","url":"funsies/schema.html#get_part_table"},{"doc":"funsies/schema","title":"funsies/schema","content":"","url":"funsies/schema.html"},{"doc":"builders/by_id","title":"render","content":"pub fn render(table table: Table) -&gt; String\n\nSynonyms:\nbuilders/by_id.render\nbuilders/by_id render","url":"builders/by_id.html#render"},{"doc":"builders/by_id","title":"render_builder","content":"pub fn render_builder(table table: Table) -&gt; StringBuilder\n\nSynonyms:\nbuilders/by_id.render_builder\nbuilders/by_id render_builder","url":"builders/by_id.html#render_builder"},{"doc":"builders/by_id","title":"builders/by_id","content":"","url":"builders/by_id.html"},{"doc":"builders/get_all","title":"render","content":"pub fn render(table table: Table) -&gt; String\n\nSynonyms:\nbuilders/get_all.render\nbuilders/get_all render","url":"builders/get_all.html#render"},{"doc":"builders/get_all","title":"render_builder","content":"pub fn render_builder(table table: Table) -&gt; StringBuilder\n\nSynonyms:\nbuilders/get_all.render_builder\nbuilders/get_all render_builder","url":"builders/get_all.html#render_builder"},{"doc":"builders/get_all","title":"builders/get_all","content":"","url":"builders/get_all.html"},{"doc":"funsies/query/yummy","title":"Eq","content":"pub type Eq {\n  Equals(col: String, value: Value)\n  Not(Eq)\n  Or(Eq, Eq)\n  And(Eq, Eq)\n}\n\nEquals(col: String, value: Value)\n\n\nNot(Eq)\n\n\nOr(Eq, Eq)\n\n\nAnd(Eq, Eq)\n\n\nSynonyms:\nfunsies/query/yummy.Eq\nfunsies/query/yummy Eq","url":"funsies/query/yummy.html#Eq"},{"doc":"funsies/query/yummy","title":"QueryBuilder","content":"pub type QueryBuilder {\n  QueryBuilder(\n    table: schema.Table,\n    select_columns: List(String),\n    where_clauses: List(Eq),\n    order_by_clauses: List(String),\n    insert_columns: List(String),\n    insert_values: List(Value),\n  )\n}\n\nQueryBuilder(\n  table: schema.Table,\n  select_columns: List(String),\n  where_clauses: List(Eq),\n  order_by_clauses: List(String),\n  insert_columns: List(String),\n  insert_values: List(Value),\n)\n\n\nSynonyms:\nfunsies/query/yummy.QueryBuilder\nfunsies/query/yummy QueryBuilder","url":"funsies/query/yummy.html#QueryBuilder"},{"doc":"funsies/query/yummy","title":"Value","content":"pub type Value {\n  IntValue(Int)\n  StringValue(String)\n  BoolValue(Bool)\n  ErrorValue(String)\n}\n\nIntValue(Int)\n\n\nStringValue(String)\n\n\nBoolValue(Bool)\n\n\nErrorValue(String)\n\n\nSynonyms:\nfunsies/query/yummy.Value\nfunsies/query/yummy Value","url":"funsies/query/yummy.html#Value"},{"doc":"funsies/query/yummy","title":"WhereBuilder","content":"pub type WhereBuilder {\n  WhereBuilder(table: schema.Table, clauses: List(Eq))\n}\n\nWhereBuilder(table: schema.Table, clauses: List(Eq))\n\n\nSynonyms:\nfunsies/query/yummy.WhereBuilder\nfunsies/query/yummy WhereBuilder","url":"funsies/query/yummy.html#WhereBuilder"},{"doc":"funsies/query/yummy","title":"and","content":"pub fn and(builder: WhereBuilder) -&gt; WhereBuilder\n\nSynonyms:\nfunsies/query/yummy.and\nfunsies/query/yummy and","url":"funsies/query/yummy.html#and"},{"doc":"funsies/query/yummy","title":"equals","content":"pub fn equals(\n  builder: WhereBuilder,\n  col: String,\n  value: a,\n) -&gt; WhereBuilder\n\nSynonyms:\nfunsies/query/yummy.equals\nfunsies/query/yummy equals","url":"funsies/query/yummy.html#equals"},{"doc":"funsies/query/yummy","title":"insert","content":"pub fn insert(\n  builder: QueryBuilder,\n  values: List(Value),\n) -&gt; QueryBuilder\n\nSynonyms:\nfunsies/query/yummy.insert\nfunsies/query/yummy insert","url":"funsies/query/yummy.html#insert"},{"doc":"funsies/query/yummy","title":"new","content":"pub fn new(table: Table) -&gt; QueryBuilder\n\nSynonyms:\nfunsies/query/yummy.new\nfunsies/query/yummy new","url":"funsies/query/yummy.html#new"},{"doc":"funsies/query/yummy","title":"not","content":"pub fn not(builder: WhereBuilder) -&gt; WhereBuilder\n\nSynonyms:\nfunsies/query/yummy.not\nfunsies/query/yummy not","url":"funsies/query/yummy.html#not"},{"doc":"funsies/query/yummy","title":"or","content":"pub fn or(builder: WhereBuilder) -&gt; WhereBuilder\n\nSynonyms:\nfunsies/query/yummy.or\nfunsies/query/yummy or","url":"funsies/query/yummy.html#or"},{"doc":"funsies/query/yummy","title":"order_by","content":"pub fn order_by(\n  builder: QueryBuilder,\n  column: String,\n  direction: String,\n) -&gt; QueryBuilder\n\nSynonyms:\nfunsies/query/yummy.order_by\nfunsies/query/yummy order_by","url":"funsies/query/yummy.html#order_by"},{"doc":"funsies/query/yummy","title":"select","content":"pub fn select(builder: QueryBuilder) -&gt; QueryBuilder\n\nSynonyms:\nfunsies/query/yummy.select\nfunsies/query/yummy select","url":"funsies/query/yummy.html#select"},{"doc":"funsies/query/yummy","title":"to_insert_sql","content":"pub fn to_insert_sql(\n  builder: QueryBuilder,\n) -&gt; Result(String, String)\n\nSynonyms:\nfunsies/query/yummy.to_insert_sql\nfunsies/query/yummy to_insert_sql","url":"funsies/query/yummy.html#to_insert_sql"},{"doc":"funsies/query/yummy","title":"to_sql","content":"pub fn to_sql(builder: QueryBuilder) -&gt; Result(String, String)\n\nSynonyms:\nfunsies/query/yummy.to_sql\nfunsies/query/yummy to_sql","url":"funsies/query/yummy.html#to_sql"},{"doc":"funsies/query/yummy","title":"w","content":"pub fn w(v: a) -&gt; Value\n\nSynonyms:\nfunsies/query/yummy.w\nfunsies/query/yummy w","url":"funsies/query/yummy.html#w"},{"doc":"funsies/query/yummy","title":"wb","content":"pub fn wb(table: Table) -&gt; WhereBuilder\n\nSynonyms:\nfunsies/query/yummy.wb\nfunsies/query/yummy wb","url":"funsies/query/yummy.html#wb"},{"doc":"funsies/query/yummy","title":"where","content":"pub fn where(\n  builder: QueryBuilder,\n  condition: WhereBuilder,\n) -&gt; QueryBuilder\n\nSynonyms:\nfunsies/query/yummy.where\nfunsies/query/yummy where","url":"funsies/query/yummy.html#where"},{"doc":"funsies/query/yummy","title":"funsies/query/yummy","content":"","url":"funsies/query/yummy.html"},{"doc":"builders/insert","title":"render","content":"pub fn render(table table: Table) -&gt; String\n\nSynonyms:\nbuilders/insert.render\nbuilders/insert render","url":"builders/insert.html#render"},{"doc":"builders/insert","title":"render_builder","content":"pub fn render_builder(table table: Table) -&gt; StringBuilder\n\nSynonyms:\nbuilders/insert.render_builder\nbuilders/insert render_builder","url":"builders/insert.html#render_builder"},{"doc":"builders/insert","title":"builders/insert","content":"","url":"builders/insert.html"},{"doc":"funsies/orm_gen","title":"create_by_id","content":"pub fn create_by_id(\n  path: String,\n  table: Table,\n) -&gt; Result(Nil, FileError)\n\nSynonyms:\nfunsies/orm_gen.create_by_id\nfunsies/orm_gen create_by_id","url":"funsies/orm_gen.html#create_by_id"},{"doc":"funsies/orm_gen","title":"create_get_all","content":"pub fn create_get_all(\n  path: String,\n  table: Table,\n) -&gt; Result(Nil, FileError)\n\nSynonyms:\nfunsies/orm_gen.create_get_all\nfunsies/orm_gen create_get_all","url":"funsies/orm_gen.html#create_get_all"},{"doc":"funsies/orm_gen","title":"create_insert","content":"pub fn create_insert(\n  path: String,\n  table: Table,\n) -&gt; Result(Nil, FileError)\n\nSynonyms:\nfunsies/orm_gen.create_insert\nfunsies/orm_gen create_insert","url":"funsies/orm_gen.html#create_insert"},{"doc":"funsies/orm_gen","title":"funsies/orm_gen","content":"","url":"funsies/orm_gen.html"},{"doc":"cli_project/project","title":"create_files","content":"pub fn create_files(\n  input: Dict(String, String),\n) -&gt; Result(Nil, FileError)\n\nSynonyms:\ncli_project/project.create_files\ncli_project/project create_files","url":"cli_project/project.html#create_files"},{"doc":"cli_project/project","title":"find_schema_files","content":"pub fn find_schema_files() -&gt; Result(Dict(String, String), Nil)\n\nSynonyms:\ncli_project/project.find_schema_files\ncli_project/project find_schema_files","url":"cli_project/project.html#find_schema_files"},{"doc":"cli_project/project","title":"name","content":"pub fn name() -&gt; Result(String, Nil)\n Returns the project&#39;s name, read from the `gleam.toml` file.\n\nSynonyms:\ncli_project/project.name\ncli_project/project name","url":"cli_project/project.html#name"},{"doc":"cli_project/project","title":"src","content":"pub fn src() -&gt; String\n Finds the path of the project&#39;s `src` directory.\n This recursively walks up from the current directory until it finds a\n `gleam.toml` and builds it from there.\n\nSynonyms:\ncli_project/project.src\ncli_project/project src","url":"cli_project/project.html#src"},{"doc":"cli_project/project","title":"work","content":"pub fn work() -&gt; Result(Nil, Nil)\n\nSynonyms:\ncli_project/project.work\ncli_project/project work","url":"cli_project/project.html#work"},{"doc":"cli_project/project","title":"cli_project/project","content":" A set of utilities to work within a Gleam project. I TOOK MOST OF THIS FROM THE SQUIRREL REPO. TYSM!\n","url":"cli_project/project.html"},{"doc":"funsies","title":"main","content":"pub fn main() -&gt; Nil\n\nSynonyms:\nfunsies.main\nfunsies main","url":"funsies.html#main"},{"doc":"funsies","title":"funsies","content":"","url":"funsies.html"},{"doc":"funsies/decoder","title":"generate_decoder_code","content":"pub fn generate_decoder_code(table: Table) -&gt; String\n\nSynonyms:\nfunsies/decoder.generate_decoder_code\nfunsies/decoder generate_decoder_code","url":"funsies/decoder.html#generate_decoder_code"},{"doc":"funsies/decoder","title":"generate_row_type","content":"pub fn generate_row_type(table: Table) -&gt; String\n\nSynonyms:\nfunsies/decoder.generate_row_type\nfunsies/decoder generate_row_type","url":"funsies/decoder.html#generate_row_type"},{"doc":"funsies/decoder","title":"funsies/decoder","content":"","url":"funsies/decoder.html"},{"doc":"funsies/query/runner","title":"run","content":"pub fn run(\n  sql: String,\n  db: Connection,\n  decoder: fn(Dynamic) -&gt; Result(a, List(DecodeError)),\n) -&gt; Result(Returned(a), QueryError)\n\nSynonyms:\nfunsies/query/runner.run\nfunsies/query/runner run","url":"funsies/query/runner.html#run"},{"doc":"funsies/query/runner","title":"funsies/query/runner","content":"","url":"funsies/query/runner.html"}]);